/* Encyklopedie jazyka C
 *
 * Kapitola Tvorba vlastních příkazů
 *
 */

/* Copyright (c) David Martinek, 2021--2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

// Takto se natahují systémové knihovny funkcí. Ty se uvádí v úhlových
// závorkách. Uživatelské knihovny se natahují pomocí dvojitých uvozovek,
// např. takto: #include "mujmodul.h".
#include <stdio.h>
#include <stdlib.h>

#define VERZE "1.7"

/** Podprogram (procedura) prikaz1.
 *
 * Podprogram je pojmenovaný kus kódu, který jde opakovaně volat z jiných míst
 * programu. Je to základní stavební prvek každého rozumného programu.
 *
 * V jazyce C se podprogramům říká funkce. Často (asi nesprávně) i takovým,
 * které nevrací žádnou hodnotu.
 *
 * Podprogram se skládá se z hlavičky a těla ve složených závorkách.
 *
 * Hlavička má formát: návratovýTyp jménoFunkce(seznamParametrů).
 *
 * Jméno funkce, respektive podprogramu je identifikátor, což v jazyce C znamená
 * že musí začínat znakem anglické abecedy nebo podtržítkem (znaky _a-zA-Z).
 * Další znaky mohou tvořit i číslice (znaky _a-zA-Z0-9). Nelze používat českou
 * diakritiku. Také nejde používat znaky jako #, +, -, /, ? atd. protože tyto
 * znaky mají v jazyce C svůj definovaný význam. V identifikátorech nejde
 * používat ani mezery. Pro víceslovní názvy funkcí je zvykem používat místo
 * mezer podtržítka (funkce_se_zbytecne_dlouhym_nazvem), nebo častěji tzv.
 * camelCase zápis, kdy první slovo začíná malým písmenem a další slova velkými
 * bez podtržítek (funkceSeZbytecneDlouhymNazvem).
 *
 * Pokud se jako návratovýTyp použije prázdný typ void, znamená to, že funkce
 * nevrací žádnou hodnotu. Vyrobili jsme tak uživatelský příkaz. Někdy se
 * takovému podprogramu také říká procedura.
 *
 * Podprogramy mohou mít parametry. Pokud se seznamParametrů vynechá, nebo
 * se použije klíčové slovo void (doporučené čistší řešení - kvůli starším
 * verzím překladače), podprogram se bude používat bez parametrů.
 *
 * Tento podprogram mohou volat všechny podprogramy, které jsou vyrobeny za ním.
 * V C platí pravidlo, že cokoli (podprogramy, proměnné) jde použít
 * až poté, co to bylo vyrobeno. Zde podprogram prikaz2 může volat podprogram
 * prikaz1, ale prikaz1 nemůže volat prikaz2, protože ho nevidí.
 *
 * Tento podprogram se použije, tedy zavolá tak, že se napíše jeho jméno
 * a kulaté závorky. Protože tento konkrétní podprogram nemá v hlavičce uvedeny
 * (deklarovány) žádné parametry, není třeba dosazovat žádné konkrétní argumenty
 * a kulaté závorky zůstanou prázdné. Od teď je tedy možné používat příkaz
 * prikaz1(); - viz funkce main.
 */
void prikaz1(void)
{
  printf("*** prikaz1 - zacatek ***\n");
  printf("Ahoj, ja jsem podprogram prikaz1.\n");
  printf("*** prikaz1 - konec ***\n");
}

/** Hlavička podprogramu prikaz2.
 *
 * Toto je druhý způsob, jak vytvářet podprogramy - před funkci main jde umístit
 * hlavička ukončená středníkem. Tomu se říká prototyp funkce/podprogramu nebo
 * také deklarace funkce/podprogramu.
 *
 * Jméno podprogramu jde používat od místa deklarace, takže tento podprogram
 * lze volat kdekoli to jde od tohoto místa až do konce zdrojového souboru.
 *
 * Tělo podprogramu, tedy jeho implementace může být někde jinde. Zde je za
 * funkcí main, ale mohlo by být i v jiném souboru (modulu).
 *
 * Tento způsob je výhodný, když
 * (a) chcete mít ve zdrojovém textu pořádek a nechcete hledat funkci main
 *     někde na konci programu,
 * (b) podprogramy se vzájemně volají - pak nemusíte řešit, který musí být
 *     napsán dříve a který později,
 * (c) vytváříte moduly a knihovny - hlavičky se pak píší do hlavičkových
 *     souborů (s koncovkou .h) a implementace se píší do zdrojových souborů
 *     (s koncovkou .c).
 */
void prikaz2(void);

////////////////////////////////////////////////////////////////////////////////
/** Funkce main je startovní bod programu.
 *
 * Když se spouští přeložený program, spustí se funkce main. Program skončí ve
 * chvíli, kdy skončí funkce main.
 *
 * Očekává se, že funkce main volá další podprogramy.
 *
 * Ve funkci main by mělo být co nejméně produkčního kódu. Ten by měl být
 * realizován v podprogramech.
 */
int main(void)
{
  /* Příkaz printf umí tisknout textové řetězce.*/
  printf(
    "========================================================================\n"
  );

  /* Umí tisknout i hodnoty proměnných, konstant a výrazů pomocí formátovacích
  značek - zde %s (dosaď textový řetězec).
  */
  printf(
    "Encyklopedie jazyka C - Kapitola 1. Tvorba vlastnich prikazů (verze %s)\n"
    , VERZE
  );

  /* Tisk víceřádkového textu nevyžaduje opakované volání příkazu printf.
  Stačí do textu doplnit znaky konců řádků \n.
  Další fintou je rozdělit textový řetězec po řádcích. V jazyce C platí, že
  dva textové řetězce oddělené jen bílými znaky (mezery, odřádkování) tvoří
  jeden řetězec.
  */
  printf(
    "(c) David Martinek, 2021--2023\n"
    "Zaroven s programem sleduj i zdrojový text!\n"
    "========================================================================\n"
    "\n"
    "- Tato kapitola obsahuje vetsinu informaci v komentarich zdrojoveho textu.\n"
    "\n"
  );


  /* Volání příkazu prikaz1. Kulaté závorky jsou povinné, i když podprogram
  nemá v hlavičce deklarovány žádné parametry - je to operátor volání
  podprogramu.
  */
  prikaz1();

  /* Podprogram prikaz2 jde tady zavolat i přesto, že jeho implementace je za
  funkcí main, protože jeho prototyp je uveden před funkcí main.
  */
  prikaz2();

  /* Příkaz return ukončuje funkci a vrací její výsledek. Když se volá ve funkci
  main, ukončuje celý program a vrací chybový kód. Argument 0 značí, že
  program končí bez chyby. Jiná hodnota sděluje, že program skončil s chybou
  - tuto hodnotu mohou využít jiné programy, které tento program zavolaly.
  Autor programu by měl v dokumentaci vysvětlit, co který chybový kód
  znamená. My budeme pro jednoduchost vracet hodnotu 0, když bude vše
  v pořádku, nebo 1 při chybě.

  Také lze použít konstanty (z knihovny stdlib.h) EXIT_SUCCESS
  a EXIT_FAILURE. Jejich použití mimo funkci main ale obvykle nemá význam.
  */
  return 0;
}

// Toto je implementace podprogramu prikaz2, jehož prototyp je před funkcí main.
void prikaz2(void)
{
  printf("*** prikaz2 - zacatek ***\n");
  printf("Ahoj, ja jsem podprogram prikaz2.\n");
  printf("Tady volam prikaz1():\n");
  prikaz1();
  printf("*** prikaz2 - konec ***\n");
}
